/*-
 * Copyright (C) 2001-2003 Juergen Keil.
 * Copyright (C) 2001-2003 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "audiohlp_convert.h"

/*
 * hide gcc2 __attribute__ compiler extension from non-gcc compiler
 * and old gcc compiler
 */
#if     !defined(__GNUC__)
#define __attribute__(x)        /**/
#elif   __GNUC__ < 2
#define __attribute__(x)        /**/
#endif


extern uint_t audiohlp_errlevel;
extern uint_t audiohlp_errmask;

/* audiohlp_errlevel */
#define	AUDL_L0	    0	/* warnings */
#define	AUDL_L1	    1	/* errors */
#define	AUDL_L2	    2	/* recoverable errors, debug only */
#define	AUDL_L3	    3	/* interesting data, debug only */
#define	AUDL_L4	    4	/* tracing, debug only */

/* audiohlp_errmask */
#define	AUDM_STRMSG 0x00000010	/* STREAMS message types */
#define	AUDM_IOCTL  0x00000008	/* audio_hlp_ioctl */
#define	AUDM_CLOSE  0x00000004	/* audio_hlp_close */
#define	AUDM_OPEN   0x00000002	/* audio_hlp_open */
#define	AUDM_MODS   0x00000001	/* _init, _info, _fini */
#define	AUDM_ALL    (~0u)

#if	DEBUG
#define	AUD_DPRINTF_L4(...)	audiohlp_dprintf4(__VA_ARGS__)
#define	AUD_DPRINTF_L3(...)	audiohlp_dprintf3(__VA_ARGS__)
#define	AUD_DPRINTF_L2(...)	audiohlp_dprintf2(__VA_ARGS__)

extern void audiohlp_dprintf4(uint_t mask, char *fmt, ...)
	__attribute__ ((format (__printf__, 2, 3)));
extern void audiohlp_dprintf3(uint_t mask, char *fmt, ...)
	__attribute__ ((format (__printf__, 2, 3)));
extern void audiohlp_dprintf2(uint_t mask, char *fmt, ...)
	__attribute__ ((format (__printf__, 2, 3)));
#else
#define	AUD_DPRINTF_L4(...)	/**/
#define	AUD_DPRINTF_L3(...)	/**/
#define	AUD_DPRINTF_L2(...)	/**/
#endif

#define	AUD_DPRINTF_L1(...)	audiohlp_dprintf1(__VA_ARGS__)
#define	AUD_DPRINTF_L0(...)	audiohlp_dprintf0(__VA_ARGS__)

extern void audiohlp_dprintf1(uint_t mask, char *fmt, ...)
	__attribute__ ((format (__printf__, 2, 3)));
extern void audiohlp_dprintf0(uint_t mask, char *fmt, ...)
	__attribute__ ((format (__printf__, 2, 3)));



/*
 * Some macros for converting beween driver instance numbers
 * and unix device driver dev_t
 */

/* make a dev_t from a driver instance number and the control-device-flag */
#define AUDIOHLP_MAKEDEV(i, min) ((i)<<2 | ((min)&3))

/* extract instance number from dev_t */
#define AUDIOHLP_INST(d)        (getminor(d)>>2)

/* extract instance specific minor number from dev_t */
#define AUDIOHLP_MINOR(d)       (getminor(d)&3)

/* true is this dev_t represents a sound control device */
#define AUDIOHLP_ISCTL(d)       (AUDIOHLP_MINOR(d) == AUDIOHLP_AUDIOCTL_MIN)


/* minor device bits in the lower two bits of a dev_t */
#define AUDIOHLP_AUDIO_MIN      0       /* the /dev/audio clone device */
#define AUDIOHLP_AUDIOCTL_MIN   1       /* /dev/audioctl */
#define AUDIOHLP_CLONE_MIN      2       /* first cloned /dev/audio device */
#define AUDIOHLP_CLONE_COUNT	2       /* max. of two cloned devices */


#define TRUE    1
#define FALSE   0

struct audio_handle {
    kmutex_t	    lock;
    kcondvar_t	    close_wait_cv;	/* notification for blocked open */
    kcondvar_t	    playback_empty_cv;	/* notification for AUDIO_DRAIN */

    dev_info_t	   *ah_dip;
    void       	   *ah_private;		/* opaque data for device dep. part */

    ah_ad_info_t   *ah_ad_info;

    uint8_t	    initialized;
    uint8_t	    drain_waiting;
    uint8_t	    openflags;
    uint8_t	    minor_inuse[AUDIOHLP_CLONE_COUNT];

    queue_t	   *ctl_q;
    queue_t	   *play_q;
    queue_t	   *record_q;

    mblk_t	   *play_mp;

    audio_info_t    audio_info;

    device_format_t device_play_format;
    device_format_t device_record_format;

    struct conv_ctx play_conv_ctx;
    struct conv_ctx record_conv_ctx;
};


/*
 * Macros for calling functions from the audio driver's ops vector
 */
#define AD_SETUP(ah,stream,flags)				\
    ((ah)->ah_ad_info->ad_entry->ad_setup(ah, stream, flags))
#define	AD_TEARDOWN(ah,stream,dir)				\
    ((ah)->ah_ad_info->ad_entry->ad_teardown(ah, stream, dir))
#define	AD_SET_CONFIG(ah, stream, command, dir, arg1, arg2)	\
    ((ah)->ah_ad_info->ad_entry->ad_set_config(ah, stream,	\
		command, dir, arg1, arg2))
#define	AD_SET_FORMAT(ah, stream, dir,	\
		      sample_rate, channels, precision, encoding) \
    ((ah)->ah_ad_info->ad_entry->ad_set_format(ah, stream,	\
		dir, sample_rate, channels, precision, encoding))
#define	AD_START_PLAY(ah, stream)				\
    ((ah)->ah_ad_info->ad_entry->ad_start_play(ah, stream))
#define	AD_PAUSE_PLAY(ah, stream)				\
    ((ah)->ah_ad_info->ad_entry->ad_pause_play(ah, stream))
#define	AD_STOP_PLAY(ah, stream)				\
    ((ah)->ah_ad_info->ad_entry->ad_stop_play(ah, stream))
#define	AD_START_RECORD(ah, stream)				\
    ((ah)->ah_ad_info->ad_entry->ad_start_record(ah, stream))
#define	AD_STOP_RECORD(ah, stream)				\
    ((ah)->ah_ad_info->ad_entry->ad_stop_record(ah, stream))
#define	AD_IOCTL(ah, channel, q, mp, error)			\
    ((ah)->ah_ad_info->ad_entry->ad_ioctl(ah, channel, q, mp, error))
#define	AD_IOCDATA(ah, channel, q, mp, error)			\
    ((ah)->ah_ad_info->ad_entry->ad_iocdata(ah, channel, q, mp, error))
#define	AD_GETDELAY(ah, stream)					\
    ((ah)->ah_ad_info->ad_entry->ad_get_delay(ah, stream))
/*
#define	AD_FILL_PLAY_BUFFER(ah, stream)				\
    do {							\
      if ((ah)->ah_ad_info->ad_entry->ad_fill_play_buffer)	\
	  (ah)->ah_ad_info->ad_entry->ad_fill_play_buffer(ah, stream); \
    } while (0)
*/

void audio_hlp_status_changed(struct audio_handle *handle);
void audio_hlp_audio_drain(queue_t *q);
int audio_hlp_ioctl(audiohdl_t handle, queue_t *q, mblk_t *mp, int *error);
int audio_hlp_iocdata(audiohdl_t handle, queue_t *q, mblk_t *mp, int *error);

int audio_hlp_audio_setdefaults(struct audio_handle *handle);

void copy_samples(void *dst, int dst_prec, int dst_enc, int dst_channels,
		  void *src, int src_prec, int src_enc, int src_channels,
		  int samples);
